@@ -95,7 +95,7 @@ SampleCpars <- function(cpars, nsim=48, silent=FALSE) {
              'maxage', 'n_age', 'CurrentYr',
              'plusgroup', 'control', 'AddIUnits', 'Data', 'MPA',
              'nareas', 'Wa', 'Wb', 'maxF', 'Sample_Area', 'Asize',
-             'Real.Data.Map')
+             'Real.Data.Map', 'SRR')
 
   cpars2 <- cpars
   cpars2[Names] <- NULL
@@ -187,7 +187,7 @@ SampleCpars <- function(cpars, nsim=48, silent=FALSE) {
   sampCpars
 }
 
-sample_unif <- function(par, cpars, Obj, nsim, altpar=NULL) {
+sample_unif <- function(par, cpars, Obj, nsim, altpar=NULL, req=TRUE) {
   if (!is.null(cpars[[par]])) {
     tt <- myrunif(nsim, 0,1) # call to runif to increment RNG
     return(cpars[[par]])
@@ -201,7 +201,8 @@ sample_unif <- function(par, cpars, Obj, nsim, altpar=NULL) {
   
   if (!is.null(altpar)) par <- altpar
   vals <- slot(Obj, par)
-  if (length(vals)<1) stop('slot ', par, ' in object class ', class(Obj), ' is missing values')
+  if (req)
+    if (length(vals)<1) stop('slot ', par, ' in object class ', class(Obj), ' is missing values')
   myrunif(nsim, vals[1], vals[2])
 }
 
@@ -238,7 +239,7 @@ SampleStockPars <- function(Stock, nsim=48, nyears=80, proyears=50, cpars=NULL,
   }
   # Get custom pars if they exist
   if (methods::is(Stock, "OM") && length(Stock@cpars) > 0 && is.null(cpars))
-    cpars <- SampleCpars(Stock@cpars, nsim)  # custom parameters exist in Stock/OM object
+    cpars <- SampleCpars(Stock@cpars, nsim, silent=!msg)  # custom parameters exist in Stock/OM object
 
   # ---- Maximum age ----
   if(!is.null(cpars$maxage)) {
@@ -318,11 +319,13 @@ SampleStockPars <- function(Stock, nsim=48, nyears=80, proyears=50, cpars=NULL,
   }
 
   # ---- Growth Parameters ----
-  Linf <- sample_unif('Linf', cpars, Stock, nsim)
+
+  Linf <- sample_unif('Linf', cpars, Stock, nsim, req=FALSE)
   Linfsd <- sample_unif('Linfsd', cpars, Stock, nsim)
-  K <- sample_unif('K', cpars, Stock, nsim)
+  K <- sample_unif('K', cpars, Stock, nsim, req=FALSE)
   Ksd <- sample_unif('Ksd', cpars, Stock, nsim)
-  t0 <- sample_unif('t0', cpars, Stock, nsim)
+  t0 <- sample_unif('t0', cpars, Stock, nsim, req=FALSE)
+  
 
   # Generate random numbers for random walk
   if (!is.null(cpars$Mrand)) {
@@ -355,6 +358,7 @@ SampleStockPars <- function(Stock, nsim=48, nyears=80, proyears=50, cpars=NULL,
     Karray <- GenerateRandomWalk(K, Ksd, nyears + proyears,
                                  nsim, Krand)
   }
+  
   if (!is.null(cpars$Agearray)) {
     Agearray <- cpars$Agearray
   } else {
@@ -572,7 +576,7 @@ SampleStockPars <- function(Stock, nsim=48, nyears=80, proyears=50, cpars=NULL,
         } else {
           noksims <- (1:nsim)[-oksims]
           ageMarray[oksims,yr] <- unlist(sapply(oksims, function(x)
-            LinInterp(Mat_age[x,, yr], y=1:n_age, 0.5)))
+            LinInterp(Mat_age[x,, yr], y=0:(n_age-1), 0.5)))
           ageMarray[noksims,yr] <- 1 # set to 1
           L50array[oksims,yr] <- unlist(sapply(oksims, function(x)
             LinInterp(Mat_age[x,,yr], y=Len_age[x, , nyears], 0.5)))
@@ -591,11 +595,15 @@ SampleStockPars <- function(Stock, nsim=48, nyears=80, proyears=50, cpars=NULL,
         }
         
         
-        if (length(oksims)<1 | length(oksims2)<1) {
-          age95array[,yr] <- 1.5 # set to 1.5 if < 1
+        if (length(oksims)<1 ) {
+          if (length(oksims)<1) {
+            # no maturity-at-age >= 0.95
+            age95array[,yr] <- maxage # set to 1.5 if < 1
+          }
+          
         } else {
           age95array[,yr] <- unlist(sapply(1:nsim, function(x)
-            LinInterp(Mat_age[x,, yr], y=1:n_age, 0.95)))
+            LinInterp(Mat_age[x,, yr], y=0:(n_age-1), 0.95)))
           L95array[,yr]<- unlist(sapply(1:nsim, function(x)
             LinInterp(Mat_age[x,,yr], y=Len_age[x, , nyears], 0.95)))
         }
