@@ -115,6 +115,8 @@ arma::mat movestockCPP(double nareas, double maxage, arma::cube mov, NumericMatr
 //' @param MPA Spatial closure by year and area
 //' @param control Integer. 1 to use q and effort to calculate F, 2 to use Fapic (apical F) and
 //' vulnerability to calculate F.
+//' @param SRRfun Optional. A stock-recruit function used if `SRrelc =3` 
+//' @param SRRpars Optional. A named list of arguments for `SRRfun`
 //' @param plusgroup Integer. Include a plus-group (1) or not (0)?
 //'
 //' @author A. Hordyk
@@ -127,7 +129,8 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
                List movc, double SRrelc, arma::vec Effind,
                double Spat_targc, double hc, NumericVector R0c, NumericVector SSBpRc,
                NumericVector aRc, NumericVector bRc, double Qc, double Fapic, double maxF,
-               arma::mat MPA, int control, double SSB0c, int plusgroup=0) {
+               arma::mat MPA, int control, double SSB0c, Function SRRfun, List SRRpars,
+               int plusgroup=0) {
 
   int n_age =maxage+1; // number of age classes (including age 0)
   arma::cube Narray(n_age, pyears, nareas, arma::fill::zeros);
@@ -160,6 +163,7 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
     Barray.subcube(0, 0, A, maxage, 0, A) = Ncurr.col(A) % WtAge.col(0);
     SSNarray.subcube(0, 0, A, maxage, 0, A) = Ncurr.col(A) % MatAge.col(0);
     SBarray.subcube(0, 0, A, maxage, 0, A) = Ncurr.col(A) % FecAge.col(0);
+    SBarray.subcube(0, 0, A, 0, 0, A) = 0; // first age class (recruits) is always immature
     VBarray.subcube(0, 0, A, maxage, 0, A) = Ncurr.col(A) % WtAge.col(0) % Vuln.col(0);
     Marray.subcube(0, 0, A, maxage, 0, A) = M_age.col(0);
     tempVec(A) = accu(VBarray.slice(A));
@@ -232,6 +236,11 @@ List popdynCPP(double nareas, double maxage, arma::mat Ncurr, double pyears,
       // most transparent form of the Ricker uses alpha and beta params
       rec(0) = PerrYr * aRc2(0) * SBtot * exp(-bR *SBtot);
     }
+    
+    if (SRrelc == 3) {
+      double rec_val = as<double>(SRRfun(SBtot, SRRpars));
+      rec(0) = PerrYr * rec_val;
+    }
 
     // Distribute recruitment across areas
     arma::vec recdist = R0c2/sum(R0c2); // distribution of R0
